/*
	QMPlay2 is a video and audio player.
	Copyright (C) 2010-2016  Błażej Szczygieł

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published
	by the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MAIN_HPP
#define MAIN_HPP

#define MUTEXWAIT_TIMEOUT 1250
#define TERMINATE_TIMEOUT (MUTEXWAIT_TIMEOUT*3)

#include <QMPlay2Core.hpp>

class VideoAdjustment;
class ShortcutHandler;
class ScreenSaver;
class IPCServer;
class VideoDock;
class MenuBar;
class QWidget;

class QMPlay2GUIClass : private QMPlay2CoreClass
{
public:
	static QMPlay2GUIClass &instance();

	static QString getPipe();
	static void saveCover(QByteArray cover);
	static void drawPixmap(QPainter &p, QWidget *w, QPixmap pixmap);

#ifdef UPDATER
	void runUpdate(const QString &);
#endif

	void setStyle();
	void loadIcons();

	QString getCurrentPth(QString pth = QString(), bool leaveFilename = false);
	void setCurrentPth(const QString &);

	void restoreGeometry(const QString &pth, QWidget *w, const QSize &def_size);

	inline QIcon getIcon(const QImage &img)
	{
		return img.isNull() ? *qmp2Pixmap : QPixmap::fromImage(img);
	}

	void updateInDockW();

	const QWidget *getVideoDock() const;

	QColor grad1, grad2, qmpTxt;
	QIcon *groupIcon, *mediaIcon, *folderIcon;

	MenuBar *menuBar;
	QWidget *mainW;
	IPCServer *pipe;
	ScreenSaver *screenSaver;
	VideoAdjustment *videoAdjustment;
	ShortcutHandler *shortcutHandler;
	bool restartApp, removeSettings;
private:
	QMPlay2GUIClass();
	~QMPlay2GUIClass();

	void deleteIcons();
};

#define QMPlay2GUI QMPlay2GUIClass::instance()

#endif
